# Copyright: 2007-2011 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

set auto_path [linsert $auto_path 0 [pwd]]

package require tcom
package require tcomexcel

# Test settings: Name of Excel file being generated.
#                Number of test rows and columns being generated.
# Note:
# The actual number of rows will be numRows+1, 
# because the first row contains header information.
# The actual number of columns will be numCols+1, 
# because the first column contains time information.
set xlsFile [file join [pwd] "testOut" "TestAdd"]
set numRows  10
set numCols   10 
set numSheets  3

# Test preparation: Generate lists with test data

for { set i 1 } { $i <= $numCols } { incr i } {
    lappend entList "Entity-$i"
}

for { set s 1 } { $s <= $numSheets } { incr s } {
    set appId [::Excel::Open]
    set workbookId [$appId ActiveWorkbook]
    if { $workbookId eq "" } {
        puts "Creating new workbook"
        set workbookId [::Excel::AddWorkbook $appId]
    }

    set worksheetId [::Excel::AddWorksheet $workbookId "Sheet-$s"]
    set cellsId [::Excel::GetCellsId $worksheetId]

    set t1 [clock clicks -milliseconds]
    for { set i 1 } { $i <= $numRows } { incr i } {
        ::Excel::SetRowValues  $cellsId $i 1 $entList
        #::Excel::SetRowValuesFast $cellsId $i 1 $entList
    }
    set t2 [clock clicks -milliseconds]
    puts "Time to fill worksheet: [expr $t2 - $t1] msec"
}
puts "Number of used rows in Sheet-$s   : [::Excel::GetNumUsedRows    $worksheetId]"
puts "Number of used columns in Sheet-$s: [::Excel::GetNumUsedColumns $worksheetId]"

set worksheetId [::Excel::AddWorksheet $workbookId "HeaderLines"]
set cellsId [::Excel::GetCellsId $worksheetId]
set startColumn 1
for { set i 1 } { $i <= $numRows } { incr i } {
    ::Excel::SetHeaderLine $cellsId $entList $i $startColumn
    incr startColumn
}
puts "Number of used rows in HeaderLines   : [::Excel::GetNumUsedRows    $worksheetId]"
puts "Number of used columns in HeaderLines: [::Excel::GetNumUsedColumns $worksheetId]"

set rowList [::Excel::GetRowValues $cellsId 1 5]
puts "Values of row 1 (starting at column 5): $rowList"

set colList [::Excel::GetColumnValues $cellsId 7 2]
puts "Values of column 7 (starting at row 2): $colList"

set num [::Excel::GetNumWorksheets $workbookId]
puts "Number of worksheets before deletion of Sheet-1: $num"
after 500

set sheetId [::Excel::GetWorksheetIdByName $workbookId "Sheet-1"]
::Excel::DeleteWorksheet $workbookId $sheetId

set num [::Excel::GetNumWorksheets $workbookId]
puts "Number of worksheets before deletion of last sheet: $num"
after 500

::Excel::DeleteWorksheetByIndex $workbookId $num

set num [::Excel::GetNumWorksheets $workbookId]
puts "Number of worksheets finally: $num"

# Append the default Excel filename extension.
append xlsFile [::Excel::GetExtString $appId]

# Delete Excel test file from maybe previous test.
catch { file delete -force $xlsFile }

::Excel::SaveAs $workbookId $xlsFile
