# Copyright: 2007-2011 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

package provide tcomexcel 0.3.1

namespace eval ::Excel {

    proc FindCell { cellsId str { startRowNum 1 } { endRowNum -1 } \
                                { startColNum 1 } { endColNum -1 } } {
        if { $endRowNum < 0 } {
            set endRowNum [::Excel::GetNumRows $cellsId]
        }
        if { $endColNum < 0 } {
            set endColNum [::Excel::GetNumColumns $cellsId]
        }
        for { set row $startRowNum } { $row <= $endRowNum } { incr row } {
            for { set col $startColNum } { $col <= $endColNum } { incr col } {
                set cellValue [::Excel::GetCellValue $cellsId $row $col]
                if { $cellValue eq $str } {
                    return [list $row $col]
                }
            }
        }
        return [list -1 -1]
    }

    proc SetHeaderLine { cellsId headerList { rowNum 1 } { startColNum 1 } } {
        set len [llength $headerList]
        SetRowValues $cellsId $rowNum $startColNum $headerList $len
        FormatHeaderLine $cellsId $startColNum [expr {$startColNum + $len}] $rowNum
    }

    proc FormatHeaderLine { cellsId startCol endCol { rowNum 1 } } {
        set header [::Excel::SelectRangeByIndex $cellsId $rowNum $startCol \
                                                         $rowNum $endCol]
        ::Excel::SetRangeHorizontalAlignment $header $::Excel::xlCenter
        ::Excel::SetRangeVerticalAlignment   $header $::Excel::xlCenter
        ::Excel::SetRangeFontBold $header
    }

    proc CopyWorksheet { fromWorksheetId toWorksheetId } {
        $fromWorksheetId Activate
	set rangeId [::Excel::SelectAll $fromWorksheetId]
        $rangeId Copy

        $toWorksheetId Activate
        $toWorksheetId Paste
    }

    proc InsertData { cellsId timeHeader timeList valsHeaderList valsList } {
        # Insert first row with header names.
        # First column contains the time header, next columns of row 1
        # contain the header of the data columns.
        SetCellValue $cellsId 1 1 $timeHeader

        set colNum 2
        foreach head $valsHeaderList {
            SetCellValue $cellsId 1 $colNum $head
            incr colNum
        }
        set lastDataColNum [expr $colNum - 1]

        # Format the header lines.
        FormatHeaderLine $cellsId 1 [expr $colNum-1]

        # Now insert the data row by row.
        set rowNum 2
        foreach t $timeList vals $valsList {
            set colNum 1
            SetCellValue $cellsId $rowNum $colNum $t
            incr colNum
            foreach val $vals {
                SetCellValue $cellsId $rowNum $colNum $val
                incr colNum
            }
            incr rowNum
        }
        set lastDataRowNum [expr $rowNum - 1]

        incr rowNum
        set r $rowNum
        set funcList [list \
            [::Excel::GetLangFuncName "MIN"] \
            [::Excel::GetLangFuncName "MAX"] \
            [::Excel::GetLangFuncName "AVERAGE"] \
            [::Excel::GetLangFuncName "STDEV"]]
        foreach labelStr $funcList {
            SetCellValue $cellsId $r 1 $labelStr
            SetRangeFontBold [::Excel::GetCellIdByIndex $cellsId $r 1] true
            incr r
        }

        for { set c 2 } { $c <= $lastDataColNum } { incr c } {
            set dataRange [::Excel::GetCellRange 2 $c $lastDataRowNum $c]
            set r $rowNum
            foreach func $funcList {
                set cell [SelectRangeByIndex $cellsId $r $c $r $c]
                set formula [format "=%s(%s)" $func $dataRange]
                $cell Formula $formula
                incr r
            }
        }
    }
}
